----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: scalp_axis_to_sp.vhd
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.1
-- Description: AXI Stream to Scalp Packet converter.
--
-- Last update: 2020-12-22
--
---------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

--use ieee.std_logic_unsigned.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_misc.all;

--library UNISIM;
--use UNISIM.VCOMPONENTS.all;

library work;
use work.axi4_pkg.all;
use work.scalp_misc.all;
use work.scalp_utility.all;

-- 31      23      15      7       0
--  0       7      15     23      31
--     8       8       8       8
-- ---------------------------------
-- | x_dst | y_dst | z_dst | type  |
-- ---------------------------------
-- | x_src | y_srd | z_src |  pad  |
-- ---------------------------------
--        16              16
-- ---------------------------------
-- | payload_size  |     pad       |
-- ---------------------------------
--
-- ---------------------------------
-- |           payload             |
-- ---------------------------------
--                .
--                .
--                .
-- ---------------------------------
-- |           payload             |
-- ---------------------------------

-- Register map
--
-- 31                              0
-- ---------------------------------
-- |             H0                | Dst addr + Type
-- ---------------------------------
-- ---------------------------------
-- |             H1                | Src addr + Pad
-- ---------------------------------
-- ---------------------------------
-- |             H2                | Pld size + Pad
-- ---------------------------------
-- ---------------------------------
-- |              P                | N x Payload
-- ---------------------------------

entity scalp_axis_to_sp is

    generic (
        C_SCALP_PACKET_PAYLOAD_SIZE : integer range 1 to C_SCALP_PACKET_LENGTH_RANGE_VALUE := 8;
        C_SCALP_RANDOM_READY        : boolean                                              := true);

    port (
        -- Sys clk and sys nrst
        SysClkxCI           : in  std_ulogic;
        SysRstxRNAI         : in  std_ulogic;
        -- Axis
        ScalpAxism2sxDI     : in  t_axi4m2s;
        ScalpAxiss2mxDO     : out t_axi4s2m;
        -- Scalp Packet
        ScalpPacketxDO      : out t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1)));
        ScalpPacketValidxSO : out std_ulogic);

end entity scalp_axis_to_sp;

architecture behavioral of scalp_axis_to_sp is

    signal ScalpAxism2sxD      : t_axi4m2s                                            := C_NO_AXI4_M2S;
    signal ScalpAxiss2mxD      : t_axi4s2m                                            := C_NO_AXI4_S2M;
    signal ScalpPacketStatexDP : t_scalp_packet_states                                := E_SP_IDLE;
    signal ScalpPacketStatexDN : t_scalp_packet_states                                := E_SP_IDLE;
    signal ScalpPacketCntxDP   : integer range 0 to C_SCALP_PACKET_LENGTH_RANGE_VALUE := 0;
    signal ScalpPacketCntxDN   : integer range 0 to C_SCALP_PACKET_LENGTH_RANGE_VALUE := 0;
    signal ScalpPacketValidxSP : std_ulogic                                           := '0';
    signal ScalpPacketValidxSN : std_ulogic                                           := '0';
    signal ScalpPacketH0xDP    : std_ulogic_vector(0 to ((C_BYTE_SIZE * 4) - 1))      := (others => '0');
    signal ScalpPacketH0xDN    : std_ulogic_vector(0 to ((C_BYTE_SIZE * 4) - 1))      := (others => '0');
    signal ScalpPacketH1xDP    : std_ulogic_vector(0 to ((C_BYTE_SIZE * 4) - 1))      := (others => '0');
    signal ScalpPacketH1xDN    : std_ulogic_vector(0 to ((C_BYTE_SIZE * 4) - 1))      := (others => '0');
    signal ScalpPacketxD       : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1)))
        := (SpHeaderxD  => C_NO_SCALP_PACKET_HEADER,
            SpPayloadxD => (others => C_NO_SCALP_PACKET_WORD));
    signal RandNumxD : integer    := 0;
    signal ReadyxS   : std_ulogic := '1';

begin  -- architecture behavioral

    EntityIOxB : block is
    begin  -- block EntityIOxB
        ScalpPacketxAS      : ScalpPacketxDO      <= ScalpPacketxD;
        ScalpPacketValidxAS : ScalpPacketValidxSO <= ScalpPacketValidxSP;
        ScalpAxism2sxAS     : ScalpAxism2sxD      <= ScalpAxism2sxDI;
        ScalpAxiss2mxAS     : ScalpAxiss2mxDO     <= ScalpAxiss2mxD;
    end block EntityIOxB;

    RandomReadyxG : if C_SCALP_RANDOM_READY = true generate

        RandomReadyxP : process is
            variable SeedB1xD        : integer := 999;
            variable SeedB2xD        : integer := 999;
            variable MinCyclesxD     : integer := 1;
            variable MaxCyclesxD     : integer := 3;
            variable WaitingCyclesxD : integer := 0;
            variable MinValRealxD    : real    := 0.0;
            variable MaxValReadxD    : real    := 1.0;
            variable RandomValuexD   : real    := 0.0;
            variable ThresholdxD     : real    := 0.3;
            variable RandxD          : real    := 0.0;

            -- Random Integer Value
            impure function scalp_rand_int (
                MinValxD : integer;
                MaxValxD : integer)
                return integer is
                variable RandxD : real;
            begin  -- function scalp_rand_int
                uniform(SeedB1xD, SeedB2xD, RandxD);
                return integer(round(RandxD * real(MaxValxD - MinValxD + 1) + real(MinValxD) - 0.5));
            end function scalp_rand_int;

            -- Random Real Value
            impure function scalp_rand_real (
                MinValxD : real;
                MaxValxD : real)
                return real is
                variable RandxD : real;
            begin  -- function scalp_rand_real
                uniform(SeedB1xD, SeedB2xD, RandxD);
                return RandxD * (MaxValxD - MinValxD) + MinValxD;
            end function scalp_rand_real;

        begin  -- process RandomReadyxP
            -- Axis ReadyxS Default Value
            ScalpAxiss2mxD.ReadyxS <= '1';
            ReadyxS                <= '1';

            if C_SCALP_RANDOM_READY = true then
                RandomValuexD := scalp_rand_real(MinValRealxD, MaxValReadxD);

                if RandomValuexD < ThresholdxD then
                    ScalpAxiss2mxD.ReadyxS <= '0';
                    ReadyxS                <= '0';
                else
                    ScalpAxiss2mxD.ReadyxS <= '1';
                    ReadyxS                <= '1';
                end if;
            end if;

            WaitingCyclesxD := scalp_rand_int(MinCyclesxD, MaxCyclesxD);

            for i in 0 to (WaitingCyclesxD - 1) loop
                wait until rising_edge(SysClkxCI);
            end loop;  -- i
        end process RandomReadyxP;

    elsif C_SCALP_RANDOM_READY = false generate

        ScalpAxiss2mReadyxAS : ScalpAxiss2mxD.ReadyxS <= '1';
        ReadyxAS             : ReadyxS                <= '1';

    end generate RandomReadyxG;

    StateMachineUpdateRegxP : process (SysClkxCI, SysRstxRNAI) is
    begin  -- process StateMachineUpdateRegxP
        if SysRstxRNAI = '0' then
            ScalpPacketStatexDP <= E_SP_H0;
            ScalpPacketCntxDP   <= 0;
            ScalpPacketValidxSP <= '0';
            ScalpPacketH0xDP    <= (others => '0');
            ScalpPacketH1xDP    <= (others => '0');
        elsif rising_edge(SysClkxCI) then
            ScalpPacketStatexDP <= ScalpPacketStatexDN;
            ScalpPacketCntxDP   <= ScalpPacketCntxDN;
            ScalpPacketValidxSP <= ScalpPacketValidxSN;
            ScalpPacketH0xDP    <= ScalpPacketH0xDN;
            ScalpPacketH1xDP    <= ScalpPacketH1xDN;
        end if;
    end process StateMachineUpdateRegxP;

    AxisToSpOutAndStatesCombinatoricxP : process (ReadyxS,
                                                  ScalpAxism2sxD.DataxD,
                                                  ScalpAxism2sxD.LastxS,
                                                  ScalpAxism2sxD.ValidxS,
                                                  ScalpPacketCntxDP,
                                                  ScalpPacketH0xDP,
                                                  ScalpPacketH1xDP,
                                                  ScalpPacketStatexDP,
                                                  ScalpPacketValidxSP) is
    begin  -- process AxisToSpOutAndStatesCombinatoricxP
        -- Default values
        -- State and counter
        ScalpPacketStatexDN <= ScalpPacketStatexDP;
        ScalpPacketCntxDN   <= ScalpPacketCntxDP;
        -- Scalp Packet Header
        ScalpPacketH0xDN    <= ScalpPacketH0xDP;
        ScalpPacketH1xDN    <= ScalpPacketH1xDP;
        -- Scalp Packet
        ScalpPacketValidxSN <= ScalpPacketValidxSP;

        case ScalpPacketStatexDP is
            when E_SP_H0 =>
                    ScalpPacketCntxDN   <= 0;
                    ScalpPacketH0xDN    <= (others => '0');
                    ScalpPacketH1xDN    <= (others => '0');
                    ScalpPacketValidxSN <= '0';

                    if ScalpAxism2sxD.ValidxS = '1' and ReadyxS = '1' then
                        ScalpPacketValidxSN <= '0';
                        ScalpPacketH0xDN    <= ScalpAxism2sxD.DataxD;
                        ScalpPacketStatexDN <= E_SP_H1;
                    end if;
            when E_SP_H1 =>
                    if ScalpAxism2sxD.ValidxS = '1' and ReadyxS = '1' then
                        ScalpPacketH1xDN    <= ScalpAxism2sxD.DataxD;
                        ScalpPacketStatexDN <= E_SP_H2;
                    end if;
            when E_SP_H2 =>
                    if ScalpAxism2sxD.ValidxS = '1' and ReadyxS = '1' then
                        ScalpPacketxD.SpHeaderxD
                            <= scalp_axis_to_sp_header_ul(ScalpPacketH0xDP, ScalpPacketH1xDP, ScalpAxism2sxD.DataxD);
                        ScalpPacketStatexDN <= E_SP_P;
                    end if;
            when E_SP_P =>
                    if ScalpAxism2sxD.ValidxS = '1' and ReadyxS = '1' then
                        ScalpPacketxD.SpPayloadxD(ScalpPacketCntxDP)
                            <= scalp_axis_to_sp_word_ul(ScalpAxism2sxD.DataxD, ScalpPacketCntxDP);
                        ScalpPacketCntxDN <= ScalpPacketCntxDP + 1;

                        if ScalpAxism2sxD.LastxS = '1' then
                            ScalpPacketValidxSN <= '1';
                            ScalpPacketStatexDN <= E_SP_H0;
                        end if;
                    end if;
            when others => null;
        end case;
    end process AxisToSpOutAndStatesCombinatoricxP;

end architecture behavioral;
