----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: scalp_sp_to_axis.vhd
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.1
-- Description: Scalp Packet to AXI Stream converter.
--
-- Last update: 2021-01-11
--
---------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--use ieee.std_logic_unsigned.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_misc.all;

--library UNISIM;
--use UNISIM.VCOMPONENTS.all;

library work;
use work.axi4_pkg.all;
use work.scalp_misc.all;
use work.scalp_utility.all;

-- 31      23      15      7       0
--  0       7      15     23      31
--     8       8       8       8
-- ---------------------------------
-- | x_dst | y_dst | z_dst | type  |
-- ---------------------------------
-- | x_src | y_srd | z_src |  pad  |
-- ---------------------------------
--        16              16
-- ---------------------------------
-- | payload_size  |     pad       |
-- ---------------------------------
--
-- ---------------------------------
-- |           payload             |
-- ---------------------------------
--                .
--                .
--                .
-- ---------------------------------
-- |           payload             |
-- ---------------------------------

-- Register map
--
-- 31                              0
-- ---------------------------------
-- |             H0                | Dst addr + Type
-- ---------------------------------
-- ---------------------------------
-- |             H1                | Src addr + Pad
-- ---------------------------------
-- ---------------------------------
-- |             H2                | Pld size + Pad
-- ---------------------------------
-- ---------------------------------
-- |              P                | N x Payload
-- ---------------------------------

entity scalp_sp_to_axis is

    generic (
        C_SCALP_PACKET_PAYLOAD_SIZE : integer range 1 to C_SCALP_PACKET_LENGTH_RANGE_VALUE := 8);

    port (
        -- Sys clk and sys nrst
        SysClkxCI           : in  std_ulogic;
        SysRstxRNAI         : in  std_ulogic;
        -- Scalp Packet
        ScalpPacketxDI      : in  t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1)));
        ScalpPacketValidxSI : in  std_ulogic;
        -- Axis
        ScalpAxism2sxDO     : out t_axi4m2s;
        ScalpAxiss2mxDI     : in  t_axi4s2m;
        -- Ready for new packet
        ScalpRdyxSO         : out std_ulogic);

end entity scalp_sp_to_axis;

architecture behavioral of scalp_sp_to_axis is

    -- Signals
    signal ScalpAxism2sxD      : t_axi4m2s                                            := C_NO_AXI4_M2S;
    signal ScalpAxiss2mxD      : t_axi4s2m                                            := C_NO_AXI4_S2M;
    signal ScalpPacketStatexDP : t_scalp_packet_states                                := E_SP_IDLE;
    signal ScalpPacketStatexDN : t_scalp_packet_states                                := E_SP_IDLE;
    signal ScalpPacketCntxDP   : integer range 0 to C_SCALP_PACKET_LENGTH_RANGE_VALUE := 0;
    signal ScalpPacketCntxDN   : integer range 0 to C_SCALP_PACKET_LENGTH_RANGE_VALUE := 0;
    signal ScalpPacketValidxS  : std_ulogic                                           := '0';
    signal ScalpPacketxD       : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1)))
        := (SpHeaderxD  => C_NO_SCALP_PACKET_HEADER,
            SpPayloadxD => (others => C_NO_SCALP_PACKET_WORD));
    -- Axis Registers
    signal DataxDP    : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    signal DataxDN    : std_ulogic_vector((C_AXI4_DATA_SIZE - 1) downto 0) := (others => '0');
    -- Others
    signal ScalpRdyxS : std_ulogic                                         := '0';

    -- Attributes
    attribute mark_debug                        : string;
    attribute keep                              : string;
    --
    -- attribute mark_debug of ScalpAxism2sxD      : signal is "true";
    -- attribute keep of ScalpAxism2sxD            : signal is "true";
    -- attribute mark_debug of ScalpAxiss2mxD      : signal is "true";
    -- attribute keep of ScalpAxiss2mxD            : signal is "true";
    -- attribute mark_debug of ScalpPacketStatexDP : signal is "true";
    -- attribute keep of ScalpPacketStatexDP       : signal is "true";
    -- attribute mark_debug of ScalpPacketStatexDN : signal is "true";
    -- attribute keep of ScalpPacketStatexDN       : signal is "true";
    -- attribute mark_debug of ScalpPacketCntxDP   : signal is "true";
    -- attribute keep of ScalpPacketCntxDP         : signal is "true";
    -- attribute mark_debug of ScalpPacketValidxS  : signal is "true";
    -- attribute keep of ScalpPacketValidxS        : signal is "true";
    -- attribute mark_debug of ScalpPacketxD       : signal is "true";
    -- attribute keep of ScalpPacketxD             : signal is "true";
    -- attribute mark_debug of DataxDP             : signal is "true";
    -- attribute keep of DataxDP                   : signal is "true";
    -- attribute mark_debug of DataxDN             : signal is "true";
    -- attribute keep of DataxDN                   : signal is "true";
    -- attribute mark_debug of ScalpRdyxS          : signal is "true";
    -- attribute keep of ScalpRdyxS                : signal is "true";

begin  -- architecture behavioral

    EntityIOxB : block is
    begin  -- block EntityIOxB
        ScalpPacketxAS      : ScalpPacketxD      <= ScalpPacketxDI;
        ScalpPacketValidxAS : ScalpPacketValidxS <= ScalpPacketValidxSI;
        ScalpAxism2sxAS     : ScalpAxism2sxDO    <= ScalpAxism2sxD;
        ScalpAxiss2mxAS     : ScalpAxiss2mxD     <= ScalpAxiss2mxDI;
        ScalpRdyxAS         : ScalpRdyxSO        <= ScalpRdyxS;
    end block EntityIOxB;

    -- Mémorisation des états
    StateMachineUpdateRegxP : process (SysClkxCI, SysRstxRNAI) is
    begin  -- process StateMachineUpdateRegxP
        if SysRstxRNAI = '0' then
            ScalpPacketStatexDP <= E_SP_IDLE;
            ScalpPacketCntxDP   <= 0;
            DataxDP             <= (others => '0');
        elsif rising_edge(SysClkxCI) then
            ScalpPacketStatexDP <= ScalpPacketStatexDN;
            ScalpPacketCntxDP   <= ScalpPacketCntxDN;
            DataxDP             <= DataxDN;
        end if;
    end process StateMachineUpdateRegxP;

    -- Combinatoire des états
    SpToAxisOutAndStatesCombinatoricxP : process (DataxDP,
                                                  ScalpAxiss2mxD.ReadyxS,
                                                  ScalpPacketCntxDP,
                                                  ScalpPacketStatexDP,
                                                  ScalpPacketValidxS,
                                                  ScalpPacketxD.SpHeaderxD,
                                                  ScalpPacketxD.SpHeaderxD.LengthxD,
                                                  ScalpPacketxD.SpPayloadxD) is
    begin  -- process SpToAxisOutAndStatesCombinatoricxP
        -- Default values
        -- State and counter
        ScalpPacketStatexDN    <= ScalpPacketStatexDP;
        ScalpPacketCntxDN      <= ScalpPacketCntxDP;
        -- Axis
        DataxDN                <= DataxDP;
        ScalpAxism2sxD.DataxD  <= (others => '0');
        ScalpAxism2sxD.KeepxD  <= (others => '1');
        ScalpAxism2sxD.LastxS  <= '0';
        ScalpAxism2sxD.ValidxS <= '0';
        ScalpRdyxS             <= '0';

        case ScalpPacketStatexDP is
            when E_SP_IDLE =>
                    ScalpRdyxS <= '1';

                    if ScalpPacketValidxS = '1' then
                        DataxDN             <= scalp_sp_header_h0_to_axis_ul(ScalpPacketxD.SpHeaderxD);
                        ScalpRdyxS          <= '0';
                        ScalpPacketStatexDN <= E_SP_H0;
                    end if;

            when E_SP_H0 =>
                    -- Extract word with convert data from little endian to big endian.
                    ScalpAxism2sxD.DataxD  <= DataxDP;
                    ScalpAxism2sxD.ValidxS <= '1';

                    if ScalpAxiss2mxD.ReadyxS = '1' then
                        DataxDN             <= scalp_sp_header_h1_to_axis_ul(ScalpPacketxD.SpHeaderxD);
                        ScalpPacketStatexDN <= E_SP_H1;
                    end if;

            when E_SP_H1 =>
                    -- Extract word with convert data from little endian to big endian.
                    ScalpAxism2sxD.DataxD  <= DataxDP;
                    ScalpAxism2sxD.ValidxS <= '1';

                    if ScalpAxiss2mxD.ReadyxS = '1' then
                        DataxDN             <= scalp_sp_header_h2_to_axis_ul(ScalpPacketxD.SpHeaderxD);
                        ScalpPacketStatexDN <= E_SP_H2;
                    end if;

            when E_SP_H2 =>
                    -- Extract word with convert data from little endian to big endian.
                    ScalpAxism2sxD.DataxD  <= DataxDP;
                    ScalpAxism2sxD.ValidxS <= '1';

                    if ScalpAxiss2mxD.ReadyxS = '1' then
                        if ScalpPacketxD.SpHeaderxD.LengthxD <= 0 then
                            ScalpPacketCntxDN   <= 0;
                            ScalpPacketStatexDN <= E_SP_IDLE;
                        else
                            DataxDN             <= scalp_sp_payload_p_to_axis_ul(ScalpPacketxD.SpPayloadxD, ScalpPacketCntxDP);
                            ScalpPacketCntxDN   <= ScalpPacketCntxDP + 1;
                            ScalpPacketStatexDN <= E_SP_P;
                        end if;
                    end if;

            when E_SP_P =>
                    ScalpAxism2sxD.DataxD  <= DataxDP;
                    ScalpAxism2sxD.ValidxS <= '1';

                    if ScalpAxiss2mxD.ReadyxS = '1' then
                        if ScalpPacketCntxDP >= (ScalpPacketxD.SpHeaderxD.LengthxD - 1) then
                            DataxDN             <= scalp_sp_payload_p_to_axis_ul(ScalpPacketxD.SpPayloadxD, ScalpPacketCntxDP);
                            ScalpPacketStatexDN <= E_SP_P_LAST;
                        else
                            DataxDN <= scalp_sp_payload_p_to_axis_ul(ScalpPacketxD.SpPayloadxD, ScalpPacketCntxDP);
                        end if;

                        ScalpPacketCntxDN <= ScalpPacketCntxDP + 1;
                    end if;

            when E_SP_P_LAST =>
                    ScalpAxism2sxD.DataxD  <= DataxDP;
                    ScalpAxism2sxD.ValidxS <= '1';
                    ScalpAxism2sxD.LastxS  <= '1';

                    if ScalpAxiss2mxD.ReadyxS = '1' then
                        ScalpPacketCntxDN   <= 0;
                        ScalpPacketStatexDN <= E_SP_IDLE;
                    end if;

            when others => null;
        end case;

    end process SpToAxisOutAndStatesCombinatoricxP;

end architecture behavioral;
