----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: tb_scalp_router_interface.vhd
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.1
-- Description: Testbench for scalp_router_interface.
--
-- Last update: 2020-02-13
--
---------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;
-- use ieee.math_real.uniform;
-- use ieee.math_real.floor;

--use ieee.std_logic_unsigned.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_misc.all;

--library UNISIM;
--use UNISIM.VCOMPONENTS.all;

library work;
use work.aurora_axi4_pkg.all;
use work.scalp_misc.all;
use work.scalp_utility.all;

entity tb_scalp_router_interface is

end entity tb_scalp_router_interface;

architecture testbench of tb_scalp_router_interface is

    constant C_CLK_PERIOD                      : time                                                 := 10 ns;
    -- Scalp Router Interface
    constant C_SCALP_INTERFACE_VECTOR_SIZE     : integer range 0 to 255                               := 7;
    constant C_SCALP_INTERFACE_SOURCE_NUMBER   : integer range 0 to 255                               := 0;
    constant C_SCALP_NEIGHBORS_VECTOR_SIZE     : integer range 0 to 255                               := 6;
    constant C_SCALP_SCHEDULER_ACK_VECTOR_SIZE : integer range 0 to 255                               := 1;
    constant C_SCALP_LOCAL_ROUTER_NET_ADDR     : t_scalp_netaddr                                      := C_3D_MIN_SCALP_NETADDR;
    constant C_SCALP_SCHEDULER_STRATEGY        : string                                               := "RR";
    -- scalp_sp_to_axis
    constant C_SCALP_PACKET_PAYLOAD_SIZE       : integer range 1 to C_SCALP_PACKET_LENGTH_RANGE_VALUE := 8;
    constant C_SCALP_RANDOM_READY              : boolean                                              := false;

    -- Scalp Packet Headers
    constant C_SP_HEADER_0 : t_scalp_packet_header
        := (DstAddrxD =>
            (XxD     => 1,
             YxD     => 1,
             ZxD     => 0),
            SrcAddrxD =>
            (XxD     => 0,
             YxD     => 0,
             ZxD     => 0),
            TypexD   => 1,
            LengthxD => C_SCALP_PACKET_PAYLOAD_SIZE);
    -- Scalp Packet Payloads
    constant C_SP_PAYLOAD_0 : t_scalp_packet_payload(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))
        := (0 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0abbccdd#, (C_BYTE_SIZE * 4))),
             IdxD   => 0),
            1 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0bccddee#, (C_BYTE_SIZE * 4))),
             IdxD   => 1),
            2 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0cddeeff#, (C_BYTE_SIZE * 4))),
             IdxD   => 2),
            3 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0deeff11#, (C_BYTE_SIZE * 4))),
             IdxD   => 3),
            4 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0eff1122#, (C_BYTE_SIZE * 4))),
             IdxD   => 4),
            5 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0f112233#, (C_BYTE_SIZE * 4))),
             IdxD   => 5),
            6 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#01223344#, (C_BYTE_SIZE * 4))),
             IdxD   => 6),
            7 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#02334455#, (C_BYTE_SIZE * 4))),
             IdxD   => 7));

    component scalp_router_interface is
        generic (
            C_SCALP_INTERFACE_VECTOR_SIZE     : integer range 0 to 255;
            C_SCALP_INTERFACE_SOURCE_NUMBER   : integer range 0 to 255;
            C_SCALP_NEIGHBORS_VECTOR_SIZE     : integer range 0 to 255;
            C_SCALP_SCHEDULER_ACK_VECTOR_SIZE : integer range 0 to 255;
            C_SCALP_SCHEDULER_STRATEGY        : string);
        port (
            SysClkxCI         : in  std_ulogic;
            SysRstxRNAI       : in  std_ulogic;
            LocNetAddrxDI     : in  t_scalp_netaddr;
            RXAxi4m2sIfxDI    : in  t_axi4m2s;
            RXAxi4s2mIfxDO    : out t_axi4s2m;
            TXAxi4m2sIfxDO    : out t_axi4m2s;
            TXAxi4s2mIfxDI    : in  t_axi4s2m;
            RXAxi4m2sLinksxDI : in  t_axi4m2s_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            RXAxi4s2mLinksxDO : out t_axi4s2m_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            TXAxi4m2sLinksxDO : out t_axi4m2s_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            TXAxi4s2mLinksxDI : in  t_axi4s2m_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            BookingVectorxDI  : in  std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            BookingVectorxDO  : out std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            SchedulerAckxDI   : in  std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            SchedulerAckxDO   : out std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            QoSVectorxDI      : in  t_scalp_qos_vector((C_SCALP_INTERFACE_VECTOR_SIZE - 1) downto 0));
    end component scalp_router_interface;

    component scalp_sp_to_axis is
        generic (
            C_SCALP_PACKET_PAYLOAD_SIZE : integer range 1 to C_SCALP_PACKET_LENGTH_RANGE_VALUE);
        port (
            SysClkxCI           : in  std_ulogic;
            SysRstxRNAI         : in  std_ulogic;
            ScalpPacketxDI      : in  t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1)));
            ScalpPacketValidxSI : in  std_ulogic;
            ScalpAxism2sxDO     : out t_axi4m2s;
            ScalpAxiss2mxDI     : in  t_axi4s2m);
    end component scalp_sp_to_axis;

    component scalp_axis_to_sp is
        generic (
            C_SCALP_PACKET_PAYLOAD_SIZE : integer range 1 to C_SCALP_PACKET_LENGTH_RANGE_VALUE;
            C_SCALP_RANDOM_READY        : boolean);
        port (
            SysClkxCI           : in  std_ulogic;
            SysRstxRNAI         : in  std_ulogic;
            ScalpAxism2sxDI     : in  t_axi4m2s;
            ScalpAxiss2mxDO     : out t_axi4s2m;
            ScalpPacketxDO      : out t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1)));
            ScalpPacketValidxSO : out std_ulogic);
    end component scalp_axis_to_sp;

    -- Sys Clk and Sys nRst
    signal SysClkxC           : std_ulogic := '0';
    signal SysRstxRNA         : std_ulogic := '0';
    -- End of Simulation
    signal EndSimxS           : boolean    := false;
    -- scalp_sp_to_axis
    signal ScalpPacketValidxS : std_ulogic := '0';
    signal ScalpPacketxD      : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))) :=
        (SpHeaderxD  => C_SP_HEADER_0,
         SpPayloadxD => C_SP_PAYLOAD_0);
    signal ScalpPacketValidEndxD : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0) := (others => '0');
    signal ScalpPacketEnd0xD     : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))) :=
        (SpHeaderxD  => C_NO_SCALP_PACKET_HEADER,
         SpPayloadxD => (others => C_NO_SCALP_PACKET_WORD));
    signal ScalpPacketEnd1xD : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))) :=
        (SpHeaderxD  => C_NO_SCALP_PACKET_HEADER,
         SpPayloadxD => (others => C_NO_SCALP_PACKET_WORD));
    signal ScalpPacketEnd2xD : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))) :=
        (SpHeaderxD  => C_NO_SCALP_PACKET_HEADER,
         SpPayloadxD => (others => C_NO_SCALP_PACKET_WORD));
    signal ScalpPacketEnd3xD : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))) :=
        (SpHeaderxD  => C_NO_SCALP_PACKET_HEADER,
         SpPayloadxD => (others => C_NO_SCALP_PACKET_WORD));
    signal ScalpPacketEnd4xD : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))) :=
        (SpHeaderxD  => C_NO_SCALP_PACKET_HEADER,
         SpPayloadxD => (others => C_NO_SCALP_PACKET_WORD));
    signal ScalpPacketEnd5xD : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))) :=
        (SpHeaderxD  => C_NO_SCALP_PACKET_HEADER,
         SpPayloadxD => (others => C_NO_SCALP_PACKET_WORD));
    -- Scalp Router Interface
    signal LocNetAddrxD       : t_scalp_netaddr                                                  := C_3D_MIN_SCALP_NETADDR;
    signal RXAxi4m2sIfxD      : t_axi4m2s                                                        := C_NO_AXI4_M2S;
    signal RXAxi4s2mIfxD      : t_axi4s2m                                                        := C_NO_AXI4_S2M;
    signal TXAxi4m2sIfxD      : t_axi4m2s                                                        := C_NO_AXI4_M2S;
    signal TXAxi4s2mIfxD      : t_axi4s2m                                                        := C_NO_AXI4_S2M;
    signal RXAxi4m2sLinksxD   : t_axi4m2s_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)   := (others => C_NO_AXI4_M2S);
    signal RXAxi4s2mLinksxD   : t_axi4s2m_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)   := (others => C_NO_AXI4_S2M);
    signal TXAxi4m2sLinksxD   : t_axi4m2s_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)   := (others => C_NO_AXI4_M2S);
    signal TXAxi4s2mLinksxD   : t_axi4s2m_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)   := (others => C_NO_AXI4_S2M);
    signal BookingVectorInxD  : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)  := (others => '0');
    signal BookingVectorOutxD : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)  := (others => '0');
    signal SchedulerAckInxD   : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)  := (others => '0');
    signal SchedulerAckOutxD  : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)  := (others => '0');
    signal QoSVectorxD        : t_scalp_qos_vector((C_SCALP_INTERFACE_VECTOR_SIZE - 1) downto 0) := (others => C_SCALP_NO_QOS);

begin  -- architecture testbench

    AsyncStatementsxB : block is
    begin  -- block AsyncStatementsxB

        SysClkxAS : SysClkxC   <= not SysClkxC after (C_CLK_PERIOD / 2);
        SysRstxAS : SysRstxRNA <= '1'          after (C_CLK_PERIOD * 10);

    end block AsyncStatementsxB;

    ScalpRouterInterfacexI : entity work.scalp_router_interface
        generic map (
            C_SCALP_INTERFACE_VECTOR_SIZE     => C_SCALP_INTERFACE_VECTOR_SIZE,
            C_SCALP_INTERFACE_SOURCE_NUMBER   => C_SCALP_INTERFACE_SOURCE_NUMBER,
            C_SCALP_NEIGHBORS_VECTOR_SIZE     => C_SCALP_NEIGHBORS_VECTOR_SIZE,
            C_SCALP_SCHEDULER_ACK_VECTOR_SIZE => C_SCALP_SCHEDULER_ACK_VECTOR_SIZE,
            C_SCALP_SCHEDULER_STRATEGY        => C_SCALP_SCHEDULER_STRATEGY)
        port map (
            SysClkxCI         => SysClkxC,
            SysRstxRNAI       => SysRstxRNA,
            LocNetAddrxDI     => LocNetAddrxD,
            RXAxi4m2sIfxDI    => RXAxi4m2sIfxD,
            RXAxi4s2mIfxDO    => RXAxi4s2mIfxD,
            TXAxi4m2sIfxDO    => TXAxi4m2sIfxD,
            TXAxi4s2mIfxDI    => TXAxi4s2mIfxD,
            RXAxi4m2sLinksxDI => RXAxi4m2sLinksxD,
            RXAxi4s2mLinksxDO => RXAxi4s2mLinksxD,
            TXAxi4m2sLinksxDO => TXAxi4m2sLinksxD,
            TXAxi4s2mLinksxDI => TXAxi4s2mLinksxD,
            BookingVectorxDI  => BookingVectorInxD,
            BookingVectorxDO  => BookingVectorOutxD,
            SchedulerAckxDI   => SchedulerAckInxD,
            SchedulerAckxDO   => SchedulerAckOutxD,
            QoSVectorxDI      => QoSVectorxD);

    ScalpSpToAxisxI : entity work.scalp_sp_to_axis
        generic map (
            C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE)
        port map (
            SysClkxCI           => SysClkxC,
            SysRstxRNAI         => SysRstxRNA,
            ScalpPacketxDI      => ScalpPacketxD,
            ScalpPacketValidxSI => ScalpPacketValidxS,
            ScalpAxism2sxDO     => RXAxi4m2sIfxD,
            ScalpAxiss2mxDI     => RXAxi4s2mIfxD);

    ScalpAxisToSp0xI : entity work.scalp_axis_to_sp
        generic map (
            C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE,
            C_SCALP_RANDOM_READY        => C_SCALP_RANDOM_READY)
        port map (
            SysClkxCI           => SysClkxC,
            SysRstxRNAI         => SysRstxRNA,
            ScalpAxism2sxDI     => TXAxi4m2sLinksxD(0),
            ScalpAxiss2mxDO     => TXAxi4s2mLinksxD(0),
            ScalpPacketxDO      => ScalpPacketEnd0xD,
            ScalpPacketValidxSO => ScalpPacketValidEndxD(0));

    ScalpAxisToSp1xI : entity work.scalp_axis_to_sp
        generic map (
            C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE,
            C_SCALP_RANDOM_READY        => C_SCALP_RANDOM_READY)
        port map (
            SysClkxCI           => SysClkxC,
            SysRstxRNAI         => SysRstxRNA,
            ScalpAxism2sxDI     => TXAxi4m2sLinksxD(1),
            ScalpAxiss2mxDO     => TXAxi4s2mLinksxD(1),
            ScalpPacketxDO      => ScalpPacketEnd1xD,
            ScalpPacketValidxSO => ScalpPacketValidEndxD(1));

    ScalpAxisToSp2xI : entity work.scalp_axis_to_sp
        generic map (
            C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE,
            C_SCALP_RANDOM_READY        => C_SCALP_RANDOM_READY)
        port map (
            SysClkxCI           => SysClkxC,
            SysRstxRNAI         => SysRstxRNA,
            ScalpAxism2sxDI     => TXAxi4m2sLinksxD(2),
            ScalpAxiss2mxDO     => TXAxi4s2mLinksxD(2),
            ScalpPacketxDO      => ScalpPacketEnd2xD,
            ScalpPacketValidxSO => ScalpPacketValidEndxD(2));

    ScalpAxisToSp3xI : entity work.scalp_axis_to_sp
        generic map (
            C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE,
            C_SCALP_RANDOM_READY        => C_SCALP_RANDOM_READY)
        port map (
            SysClkxCI           => SysClkxC,
            SysRstxRNAI         => SysRstxRNA,
            ScalpAxism2sxDI     => TXAxi4m2sLinksxD(3),
            ScalpAxiss2mxDO     => TXAxi4s2mLinksxD(3),
            ScalpPacketxDO      => ScalpPacketEnd3xD,
            ScalpPacketValidxSO => ScalpPacketValidEndxD(3));

    ScalpAxisToSp4xI : entity work.scalp_axis_to_sp
        generic map (
            C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE,
            C_SCALP_RANDOM_READY        => C_SCALP_RANDOM_READY)
        port map (
            SysClkxCI           => SysClkxC,
            SysRstxRNAI         => SysRstxRNA,
            ScalpAxism2sxDI     => TXAxi4m2sLinksxD(4),
            ScalpAxiss2mxDO     => TXAxi4s2mLinksxD(4),
            ScalpPacketxDO      => ScalpPacketEnd4xD,
            ScalpPacketValidxSO => ScalpPacketValidEndxD(4));

    ScalpAxisToSp5xI : entity work.scalp_axis_to_sp
        generic map (
            C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE,
            C_SCALP_RANDOM_READY        => C_SCALP_RANDOM_READY)
        port map (
            SysClkxCI           => SysClkxC,
            SysRstxRNAI         => SysRstxRNA,
            ScalpAxism2sxDI     => TXAxi4m2sLinksxD(5),
            ScalpAxiss2mxDO     => TXAxi4s2mLinksxD(5),
            ScalpPacketxDO      => ScalpPacketEnd5xD,
            ScalpPacketValidxSO => ScalpPacketValidEndxD(5));

    EndSimxP : process is
    begin  -- process EndSimxP
        if EndSimxS = true then
            assert false report "End of simulation!" severity failure;
        end if;

        wait until rising_edge(SysClkxC);
    end process EndSimxP;

    BookingVectorxP : process is
    begin  -- process BookingVectorxP

        wait until (BookingVectorOutxD(0) = '1' or
                    BookingVectorOutxD(1) = '1' or
                    BookingVectorOutxD(2) = '1' or
                    BookingVectorOutxD(3) = '1' or
                    BookingVectorOutxD(4) = '1' or
                    BookingVectorOutxD(5) = '1');
        wait;
    end process BookingVectorxP;

    ScalpRouterInterfacexP : process is
    begin  -- process ScalpRouterInterfacexP

        wait until (SysRstxRNA = '1');
        wait until rising_edge(SysClkxC);
        wait until rising_edge(SysClkxC);

        ScalpPacketValidxS <= '0';

        for i in 0 to 5 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        ScalpPacketValidxS <= '1';

        for i in 0 to 0 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        ScalpPacketValidxS <= '0';

        wait until (ScalpPacketValidEndxD(0) = '1' and
                    ScalpPacketValidEndxD(1) = '1' and
                    ScalpPacketValidEndxD(2) = '1' and
                    ScalpPacketValidEndxD(3) = '1' and
                    ScalpPacketValidEndxD(4) = '1' and
                    ScalpPacketValidEndxD(5) = '1');

        EndSimxS <= true;

    end process ScalpRouterInterfacexP;

end architecture testbench;
