----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: tb_scalp_sp_to_axis.vhd
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.1
-- Description: Testbench for scalp_sp_to_axis module.
--
-- Last update: 2020-02-26
--
---------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--use ieee.std_logic_unsigned.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_misc.all;

--library UNISIM;
--use UNISIM.VCOMPONENTS.all;

library work;
use work.aurora_axi4_pkg.all;
use work.scalp_misc.all;
use work.scalp_utility.all;

-- 31      23      15      7       0
--     8       8       8       8
-- ---------------------------------
-- | x_dst | y_dst | z_dst | type  |
-- ---------------------------------
-- | x_src | y_srd | z_src |  pad  |
-- ---------------------------------
--        16              16
-- ---------------------------------
-- | payload_size  |     pad       |
-- ---------------------------------
--
-- ---------------------------------
-- |           payload             |
-- ---------------------------------
--                .
--                .
--                .
-- ---------------------------------
-- |           payload             |
-- ---------------------------------

-- Register map
--
-- 31                              0
-- ---------------------------------
-- |             H0                | Dst addr + Type
-- ---------------------------------
-- ---------------------------------
-- |             H1                | Src addr + Pad
-- ---------------------------------
-- ---------------------------------
-- |             H2                | Pld size + Pad
-- ---------------------------------
-- ---------------------------------
-- |              P                | N x Payload
-- ---------------------------------

entity tb_scalp_sp_to_axis is

end entity tb_scalp_sp_to_axis;

architecture testbench of tb_scalp_sp_to_axis is

    constant C_CLK_PERIOD                : time                                                 := 10 ns;
    constant C_SCALP_PACKET_PAYLOAD_SIZE : integer range 1 to C_SCALP_PACKET_LENGTH_RANGE_VALUE := 8;
    constant C_SCALP_RANDOM_READY        : boolean                                              := true;
    constant C_SCALP_DELAY_SIZE          : integer range 0 to 255                               := 10;

    component scalp_sp_to_axis is
        generic (
            C_SCALP_PACKET_PAYLOAD_SIZE : integer range 1 to C_SCALP_PACKET_LENGTH_RANGE_VALUE);
        port (
            SysClkxCI           : in  std_ulogic;
            SysRstxRNAI         : in  std_ulogic;
            ScalpPacketxDI      : in  t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1)));
            ScalpPacketValidxSI : in  std_ulogic;
            ScalpAxism2sxDO     : out t_axi4m2s;
            ScalpAxiss2mxDI     : in  t_axi4s2m;
            ScalpRdyxSO         : out std_ulogic);
    end component scalp_sp_to_axis;

    component scalp_axis_to_sp is
        generic (
            C_SCALP_PACKET_PAYLOAD_SIZE : integer range 1 to C_SCALP_PACKET_LENGTH_RANGE_VALUE;
            C_SCALP_RANDOM_READY        : boolean);
        port (
            SysClkxCI           : in  std_ulogic;
            SysRstxRNAI         : in  std_ulogic;
            ScalpAxism2sxDI     : in  t_axi4m2s;
            ScalpAxiss2mxDO     : out t_axi4s2m;
            ScalpPacketxDO      : out t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1)));
            ScalpPacketValidxSO : out std_ulogic);
    end component scalp_axis_to_sp;

    -- Signals
    signal SysClkxC           : std_ulogic := '0';
    signal SysRstxRNA         : std_ulogic := '0';
    signal ScalpPacketValidxS : std_ulogic := '0';
    signal ScalpAxism2sxD     : t_axi4m2s  := C_NO_AXI4_M2S;
    signal ScalpAxiss2mxD     : t_axi4s2m  := C_NO_AXI4_S2M;
    signal EndSimxS           : boolean    := false;
    signal ScalpPacketOutxD   : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1)))
        := (SpHeaderxD  => C_NO_SCALP_PACKET_HEADER,
            SpPayloadxD => (others => C_NO_SCALP_PACKET_WORD));
    signal ScalpPacketValidOutxS : std_ulogic := '0';
    signal ScalpRdyxS            : std_ulogic := '0';

    -- Scalp Packets Header
    constant C_SP_HEADER_0 : t_scalp_packet_header
        := (DstAddrxD =>
            (XxD     => 1,
             YxD     => 2,
             ZxD     => 3),
            SrcAddrxD =>
            (XxD     => 4,
             YxD     => 5,
             ZxD     => 6),
            TypexD   => 1,
            LengthxD => C_SCALP_PACKET_PAYLOAD_SIZE);
    -- Scalp Packets Payload
    constant C_SP_PAYLOAD_0 : t_scalp_packet_payload(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))
        := (0 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0abbccdd#, (C_BYTE_SIZE * 4))),
             IdxD   => 0),
            1 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0bccddee#, (C_BYTE_SIZE * 4))),
             IdxD   => 1),
            2 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0cddeeff#, (C_BYTE_SIZE * 4))),
             IdxD   => 2),
            3 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0deeff11#, (C_BYTE_SIZE * 4))),
             IdxD   => 3),
            4 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0eff1122#, (C_BYTE_SIZE * 4))),
             IdxD   => 4),
            5 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0f112233#, (C_BYTE_SIZE * 4))),
             IdxD   => 5),
            6 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#01223344#, (C_BYTE_SIZE * 4))),
             IdxD   => 6),
            7 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#02334455#, (C_BYTE_SIZE * 4))),
             IdxD   => 7));
    -- Scalp Packets
    signal ScalpPacketxD : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1)))
        := (SpHeaderxD  => C_SP_HEADER_0,
            SpPayloadxD => C_SP_PAYLOAD_0);

begin  -- architecture testbench

    SimCtrlxB : block is
    begin  -- block SimCtrlxB
        SysClkxC   <= not SysClkxC after (C_CLK_PERIOD / 2);
        SysRstxRNA <= '1'          after (C_CLK_PERIOD * 10);
    end block SimCtrlxB;

    EndSimxP : process is
    begin  -- process EndSimxP
        if EndSimxS = true then
            assert false report "End of simulation!" severity failure;
        end if;

        wait until rising_edge(SysClkxC);
    end process EndSimxP;

    ScalpSpToAxisxI : entity work.scalp_sp_to_axis
        generic map (
            C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE)
        port map (
            SysClkxCI           => SysClkxC,
            SysRstxRNAI         => SysRstxRNA,
            ScalpPacketxDI      => ScalpPacketxD,
            ScalpPacketValidxSI => ScalpPacketValidxS,
            ScalpAxism2sxDO     => ScalpAxism2sxD,
            ScalpAxiss2mxDI     => ScalpAxiss2mxD,
            ScalpRdyxSO         => ScalpRdyxS);

    ScalpAxisToSpxI : entity work.scalp_axis_to_sp
        generic map (
            C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE,
            C_SCALP_RANDOM_READY        => C_SCALP_RANDOM_READY)
        port map (
            SysClkxCI           => SysClkxC,
            SysRstxRNAI         => SysRstxRNA,
            ScalpAxism2sxDI     => ScalpAxism2sxD,
            ScalpAxiss2mxDO     => ScalpAxiss2mxD,
            ScalpPacketxDO      => ScalpPacketOutxD,
            ScalpPacketValidxSO => ScalpPacketValidOutxS);

    ScalpPacketReqxP : process is
    begin  -- process ScalpPacketReqxP

        wait until (SysRstxRNA = '1');
        wait until rising_edge(SysClkxC);
        wait until rising_edge(SysClkxC);

        ScalpPacketValidxS <= '0';

        for i in 0 to 5 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        ScalpPacketValidxS <= '1';

        for i in 0 to 0 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        ScalpPacketValidxS <= '0';

        wait until (ScalpPacketValidOutxS = '1');

        for i in 0 to 5 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        -- Dst Addr
        assert ScalpPacketxD.SpHeaderxD.DstAddrxD.XxD = ScalpPacketOutxD.SpHeaderxD.DstAddrxD.XxD
            report "Error : DstAddr XxD " &
            integer'image(ScalpPacketxD.SpHeaderxD.DstAddrxD.XxD) &
            " /= " &
            integer'image(ScalpPacketOutxD.SpHeaderxD.DstAddrxD.XxD) severity failure;
        report "Valid : DstAddr XxD " &
            integer'image(ScalpPacketxD.SpHeaderxD.DstAddrxD.XxD) &
            " /= " &
            integer'image(ScalpPacketOutxD.SpHeaderxD.DstAddrxD.XxD) severity note;

        assert ScalpPacketxD.SpHeaderxD.DstAddrxD.YxD = ScalpPacketOutxD.SpHeaderxD.DstAddrxD.YxD
            report "Error : DstAddr YxD " &
            integer'image(ScalpPacketxD.SpHeaderxD.DstAddrxD.YxD) &
            " /= " &
            integer'image(ScalpPacketOutxD.SpHeaderxD.DstAddrxD.YxD) severity failure;
        report "Valid : DstAddr YxD " &
            integer'image(ScalpPacketxD.SpHeaderxD.DstAddrxD.YxD) &
            " /= " &
            integer'image(ScalpPacketOutxD.SpHeaderxD.DstAddrxD.YxD) severity note;

        assert ScalpPacketxD.SpHeaderxD.DstAddrxD.ZxD = ScalpPacketOutxD.SpHeaderxD.DstAddrxD.ZxD
            report "Error : DstAddr ZxD " &
            integer'image(ScalpPacketxD.SpHeaderxD.DstAddrxD.ZxD) &
            " /= " &
            integer'image(ScalpPacketOutxD.SpHeaderxD.DstAddrxD.ZxD) severity failure;
        report "Valid : DstAddr ZxD " &
            integer'image(ScalpPacketxD.SpHeaderxD.DstAddrxD.ZxD) &
            " /= " &
            integer'image(ScalpPacketOutxD.SpHeaderxD.DstAddrxD.ZxD) severity note;

        -- Src Addr
        assert ScalpPacketxD.SpHeaderxD.SrcAddrxD.XxD = ScalpPacketOutxD.SpHeaderxD.SrcAddrxD.XxD
            report "Error : SrcAddr XxD " &
            integer'image(ScalpPacketxD.SpHeaderxD.SrcAddrxD.XxD) &
            " /= " &
            integer'image(ScalpPacketOutxD.SpHeaderxD.SrcAddrxD.XxD) severity failure;
        report "Valid : SrcAddr XxD " &
            integer'image(ScalpPacketxD.SpHeaderxD.SrcAddrxD.XxD) &
            " /= " &
            integer'image(ScalpPacketOutxD.SpHeaderxD.SrcAddrxD.XxD) severity note;

        assert ScalpPacketxD.SpHeaderxD.SrcAddrxD.YxD = ScalpPacketOutxD.SpHeaderxD.SrcAddrxD.YxD
            report "Error : SrcAddr YxD " &
            integer'image(ScalpPacketxD.SpHeaderxD.SrcAddrxD.YxD) &
            " /= " &
            integer'image(ScalpPacketOutxD.SpHeaderxD.SrcAddrxD.YxD) severity failure;
        report "Valid : SrcAddr YxD " &
            integer'image(ScalpPacketxD.SpHeaderxD.SrcAddrxD.YxD) &
            " /= " &
            integer'image(ScalpPacketOutxD.SpHeaderxD.SrcAddrxD.YxD) severity note;

        assert ScalpPacketxD.SpHeaderxD.SrcAddrxD.ZxD = ScalpPacketOutxD.SpHeaderxD.SrcAddrxD.ZxD
            report "Error : SrcAddr ZxD " &
            integer'image(ScalpPacketxD.SpHeaderxD.SrcAddrxD.ZxD) &
            " /= " &
            integer'image(ScalpPacketOutxD.SpHeaderxD.SrcAddrxD.ZxD) severity failure;
        report "Valid : SrcAddr ZxD " &
            integer'image(ScalpPacketxD.SpHeaderxD.SrcAddrxD.ZxD) &
            " /= " &
            integer'image(ScalpPacketOutxD.SpHeaderxD.SrcAddrxD.ZxD) severity note;

        -- Type
        assert ScalpPacketxD.SpHeaderxD.TypexD = ScalpPacketOutxD.SpHeaderxD.TypexD
            report "Error : TypexD " &
            integer'image(ScalpPacketxD.SpHeaderxD.TypexD) &
            " /= " &
            integer'image(ScalpPacketOutxD.SpHeaderxD.TypexD) severity failure;
        report "Valid : TypexD " &
            integer'image(ScalpPacketxD.SpHeaderxD.TypexD) &
            " /= " &
            integer'image(ScalpPacketOutxD.SpHeaderxD.TypexD) severity note;

        -- Length
        assert ScalpPacketxD.SpHeaderxD.LengthxD = ScalpPacketOutxD.SpHeaderxD.LengthxD
            report "Error : LengthxD " &
            integer'image(ScalpPacketxD.SpHeaderxD.LengthxD) &
            " /= " &
            integer'image(ScalpPacketOutxD.SpHeaderxD.LengthxD) severity failure;
        report "Valid : LengthxD " &
            integer'image(ScalpPacketxD.SpHeaderxD.LengthxD) &
            " /= " &
            integer'image(ScalpPacketOutxD.SpHeaderxD.LengthxD) severity note;

        for i in 0 to (ScalpPacketxD.SpHeaderxD.LengthxD - 1) loop
            -- Length
            assert ScalpPacketxD.SpPayloadxD(i).WordxD = ScalpPacketOutxD.SpPayloadxD(i).WordxD
                report "Error : Payload(" & integer'image(i) & ") " &
                str_ul(ScalpPacketxD.SpPayloadxD(i).WordxD) &
                " /= " &
                str_ul(ScalpPacketOutxD.SpPayloadxD(i).WordxD) severity failure;
            report "Valid : Payload(" & integer'image(i) & ") " &
                str_ul(ScalpPacketxD.SpPayloadxD(i).WordxD) &
                " /= " &
                str_ul(ScalpPacketOutxD.SpPayloadxD(i).WordxD) severity note;
        end loop;  -- i

        EndSimxS <= true;
    end process ScalpPacketReqxP;

end architecture testbench;
